%
% Show the pdfs that compose the model
%
% l1_rng, l2_rng, r_rng are the ranges over the labels and relations to
% show. [] means to use the full range
%
% Examples:
%   model_show(model, 1, 1, 1);
%   model_show(model, 1, 1:3, 1);
%   model_show(model, 1, 1, []);
% 
% if 'imgdir' is not empty, the plots are saved to the directory instead
% 
%   model_show(model, [], [], [], 'dir') saves all the plots to the
%   directory 'dir'
%
function model_ushow(model, lbl_rng, r_rng, imgdir)

    if (~exist('lbl_rng', 'var')) || isempty(lbl_rng)
        lbl_rng = 1:model.max_labels;
    end
    if (~exist('r_rng', 'var')) || isempty(r_rng)
        r_rng = 1:model.umax_rels;
    end
    if exist('imgdir', 'var')
        save_img = 1;
    else
        save_img = 0;
    end

    if save_img
        fh = figure;
    end
    index = 1;
    for l1 = lbl_rng
        for r = r_rng
            %plot(model.pdf{l1,l2,r});
            if ~isempty(model.updf{l1,r})
                if ~save_img
                    figure; 
                else
                    clf;
                end
                N = 200;
                pts = getPoints(model.updf{l1,r});
                mn = min(pts);
                mx = max(pts);
                range = [mn, mx];
                if mn == mx
                    range(1) = mn - .05;
                    range(2) = mn + .05;
                else
                    range(1) = range(1) - .05*(range(2)-range(1));
                    range(2) = range(2) + .05*(range(2)-range(1));
                end
                x = linspace(range(1), range(2), N);
                y = evaluate(model.updf{l1,r}, x);
                h = plot(x, y, 'b-');
                set(h, 'LineWidth', 2);
                hold on;
                if 1
                for j = 1:size(pts, 2)
                    h = plot([pts(j) pts(j)], [0 evaluate(model.updf{l1,r}, pts(j))], '-r');
                    set(h, 'UserData', [model.usi{l1,r}(j) model.upi{l1,r}(j)]);
                    %patch([pts(j) pts(j) pts(j)], [0 0 evaluate(model.pdf{r}, pts(j))], 'r', 'EdgeAlpha', 0.1, 'EdgeColor', [1 0 0]);
                end
                end
                %title([clean_name(model.names{l1,l2,r}) ' (' num2str(size(pts, 2)) ' points)']);
                title(['extent ' num2str(r) ' label ' num2str(l1) ' (' num2str(size(pts, 2)) ' points)']);
                
                if 0
                [nb,xb] = hist(pts, 100);
                my = ceil(max(y));
                %nb(nb > my) = my;
                %nb = nb ./ sum(nb);
                bh = bar(xb, nb);
                set(bh, 'facecolor', 'none');
                end

                if save_img
                    %filename = [imgdir filesep 'rel_' num2str(index, '%03d') '-' model.names{l1,l2,r} '.png'];
                    filename = [imgdir filesep 'rel_' num2str(index, '%03d') '-' 'extent' num2str(r) '-label' num2str(l1) '.png'];
                    saveas(gcf, filename, 'png');
                end

                dcm_obj = datacursormode(gcf);
                set(dcm_obj, 'UpdateFcn', @myupdatefcn);
            end
            index = index + 1;
        end
    end
    if save_img
        close(fh);
    end
end


function output_txt = myupdatefcn(~, event_obj)
    pos = get(event_obj, 'Position');
    target = get(event_obj, 'Target');
    dt = get(target, 'UserData');
    output_txt = {['X: ' num2str(pos(1))]; num2str(dt)};
end
